/// <reference types="chai" />
import { Handle } from '@dojo/interfaces/core';
import Task from '@dojo/core/async/Task';
import Suite from '../Suite';
import Test from '../Test';
import ErrorFormatter, { ErrorFormatOptions } from '../common/ErrorFormatter';
import Reporter, { ReporterOptions } from '../reporters/Reporter';
import { ObjectInterface } from '../interfaces/object';
import { TddInterface } from '../interfaces/tdd';
import { BddInterface } from '../interfaces/bdd';
import { BenchmarkInterface } from '../interfaces/benchmark';
import { BenchmarkReporterOptions } from '../reporters/Benchmark';
import { RuntimeEnvironment } from '../types';
export default abstract class Executor<E extends Events = Events, C extends Config = Config, P extends Plugins = Plugins> {
    protected _assertions: {
        [name: string]: any;
    };
    protected _config: C;
    protected _rootSuite: Suite;
    protected _events: Event<E>[];
    protected _errorFormatter: ErrorFormatter;
    protected _hasSuiteErrors: boolean;
    protected _hasTestErrors: boolean;
    protected _interfaces: {
        [name: string]: any;
    };
    protected _loader: Loader;
    protected _loaderOptions: any;
    protected _loaderInit: Promise<Loader>;
    protected _loadingPlugins: {
        name: string;
        init: Task<void>;
    }[];
    protected _loadingPluginOptions: any | undefined;
    protected _listeners: {
        [event: string]: Listener<any>[];
    };
    protected _plugins: {
        [name: string]: any;
    };
    protected _reporters: Reporter[];
    protected _runTask: Task<void>;
    protected _reportersInitialized: boolean;
    constructor(options?: {
        [key in keyof C]?: any;
    });
    readonly abstract environment: RuntimeEnvironment;
    readonly config: C;
    readonly suites: Suite[];
    formatError(error: Error, options?: ErrorFormatOptions): string;
    abstract loadScript(script: string | string[]): Task<void>;
    addSuite(factory: (parentSuite: Suite) => void): void;
    configure(options: {
        [key in keyof C]?: any;
    }): void;
    emit(eventName: 'afterRun'): Task<void>;
    emit(eventName: 'beforeRun'): Task<void>;
    emit(eventName: 'runStart'): Task<void>;
    emit(eventName: 'runEnd'): Task<void>;
    emit<T extends keyof E>(eventName: T, data: E[T]): Task<void>;
    getInterface(name: 'object'): ObjectInterface;
    getInterface(name: 'tdd'): TddInterface;
    getInterface(name: 'bdd'): BddInterface;
    getInterface(name: 'benchmark'): BenchmarkInterface;
    getPlugin<Y extends keyof P>(type: Y, name: string): P[Y];
    getPlugin(name: 'chai'): Chai.ChaiStatic;
    getPlugin(name: 'interface.object'): ObjectInterface;
    getPlugin(name: 'interface.tdd'): TddInterface;
    getPlugin(name: 'interface.bdd'): BddInterface;
    getPlugin(name: 'interface.benchmark'): BenchmarkInterface;
    getPlugin<T>(name: string): T;
    log(...args: any[]): Task<void>;
    on<T extends keyof E>(eventName: T, listener: Listener<E[T]>): Handle;
    on(listener: Listener<{
        name: string;
        data?: any;
    }>): Handle;
    registerInterface(name: string, iface: any): void;
    registerLoader(init: LoaderInit): void;
    registerPlugin<T extends keyof P>(type: T, name: string, init: PluginInitializer<P[T]>): void;
    registerPlugin(name: string, init: PluginInitializer): void;
    registerReporter(name: string, Ctor: typeof Reporter): void;
    run(): Task<void>;
    protected _afterRun(): Task<void>;
    protected _assignPlugin(name: string, plugin: any): void;
    protected _beforeRun(): Task<boolean>;
    protected _initReporters(): Task<void>;
    protected _drainEventQueue(): Task<void>;
    protected _emitCoverage(source?: string): Task<void> | undefined;
    protected _evalProperty(key: string): EvaluatedProperty<C>;
    protected _loadLoader(): Promise<void>;
    protected _loadPluginsWithLoader(): Task<void>;
    protected _loadPlugins(): Task<void>;
    protected _loadScripts(scripts: PluginDescriptor[], loader: (script: string) => Promise<void>): Task<void>;
    protected _loadSuites(): Task<void>;
    protected _processOption(name: keyof C, value: any, addToExisting: boolean): void;
    protected _setOption(name: keyof C, value: any, addToExisting?: boolean, config?: C): void;
    protected _resolveConfig(): Task<void>;
    protected _runTests(): Task<void>;
}
export interface Event<E extends Events = Events> {
    eventName: keyof E;
    data?: any;
}
export interface ExecutorConstructor<E extends Events, C extends Config, T extends Executor<E, C>> {
    new (config?: Partial<C>): T;
}
export { Handle };
export interface BenchmarkConfig extends BenchmarkReporterOptions {
    id: string;
}
export interface ResourceConfig {
    loader: LoaderDescriptor;
    reporters: ReporterDescriptor[];
    plugins: PluginDescriptor[];
    suites: string[];
}
export interface Config extends ResourceConfig {
    bail: boolean;
    baseline: boolean;
    basePath: string;
    benchmark: boolean;
    benchmarkConfig?: BenchmarkConfig;
    browser: ResourceConfig;
    coverageVariable: string;
    debug: boolean;
    defaultTimeout: number;
    description: string;
    filterErrorStack: boolean;
    grep: RegExp;
    internPath: string;
    name: string;
    node: ResourceConfig;
    sessionId: string;
    showConfig: boolean;
}
export interface ReporterDescriptor {
    name: string;
    options?: ReporterOptions;
}
export interface PluginDescriptor {
    script: string;
    useLoader?: boolean;
    options?: any;
}
export interface Listener<T> {
    (arg: T): void | Promise<void>;
}
export interface CoverageMessage {
    sessionId?: string;
    source?: string;
    coverage: any;
}
export interface DeprecationMessage {
    original: string;
    replacement?: string;
    message?: string;
}
export interface ExecutorEvent {
    name: keyof Events;
    data: any;
}
export interface Events {
    '*': ExecutorEvent;
    afterRun: never;
    beforeRun: never;
    coverage: CoverageMessage;
    deprecated: DeprecationMessage;
    error: Error;
    log: string;
    runEnd: never;
    runStart: never;
    suiteAdd: Suite;
    suiteEnd: Suite;
    suiteStart: Suite;
    testAdd: Test;
    testEnd: Test;
    testStart: Test;
    warning: string;
}
export interface Plugins {
    reporter: typeof Reporter;
}
export interface Loader {
    (modules: string[]): Promise<void>;
}
export interface LoaderInit {
    (options: {
        [key: string]: any;
    }): Promise<Loader> | Loader;
}
export interface LoaderDescriptor {
    script: string;
    options?: {
        [key: string]: any;
    };
}
export interface PluginInitializer<T extends any = any> {
    (options?: {
        [key: string]: any;
    }): Task<T> | T;
}
export interface EvaluatedProperty<C extends Config = Config> {
    name: keyof C;
    addToExisting: boolean;
}
